const fs = require('fs');
const path = require('path');
const minimist = require('minimist');
const webpackConfig = require('./webpack.config');
const templating = require('./templates');
const { watch } = require('chokidar');
const { debounce } = require('lodash');

/** @type {Object} */
const argv = minimist(process.argv);

function buildIndexHtml() {
  const html = templating.compile(!!argv.dev);

  fs.writeFileSync(path.resolve(webpackConfig.output.path, 'index.html'), html);
  console.log('compiled index.html');
}

const debouncedBuildIndexHtml = debounce(buildIndexHtml, 100, { leading: true, trailing: false });

function build() {
  if (argv.watch) {
    const templatesWatcher = watch('./templates/**/*');
    templatesWatcher.on('all', debouncedBuildIndexHtml);
  } else {
    buildIndexHtml();
  }
}

build();
