import { app, BrowserWindow } from 'electron';
import os from 'os';

import './main/controllers/api';
import './main/services/database';
import * as session from './main/services/session';
import BrowserWindowConstructorOptions = Electron.BrowserWindowConstructorOptions;

export let mainWindow: Electron.BrowserWindow;

async function createWindow(): Promise<void> {
  session.setHeaders();

  // universal options
  let options: BrowserWindowConstructorOptions = {
    width: 1600,
    height: 900,
    webPreferences: {
      nodeIntegration: true,
    },
  };

  // platform specifics
  switch (os.platform()) {
    case 'win32':
      options = {
        ...options,
        ...{
          icon: 'resources/icon.ico',
        },
      };
      break;
    default:
      break;
  }

  // Create the browser window.
  mainWindow = new BrowserWindow(options);

  // and load the index.html of the app.
  await mainWindow.loadFile('frontend/index.html');

  // Open the DevTools.
  mainWindow.webContents.openDevTools();

  // Emitted when the window is closed.
  mainWindow.on('closed', () => {
    // Dereference the window object, usually you would store windows
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    mainWindow = null;
  });
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', createWindow);

// Quit when all windows are closed.
app.on('window-all-closed', () => {
  // On OS X it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

app.on('activate', async () => {
  // On OS X it"s common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (mainWindow === null) {
    await createWindow();
  }
});

// In this file you can include the rest of your app"s specific main process
// code. You can also put them in separate files and require them here.
